import React, { useState, useEffect } from 'react'
import { usePicker } from '../context.js'
import { getHandleValue } from '../utils/utils.js'

const Opacity = () => {
  const { handleChange, hc = {}, squareWidth, defaultStyles } = usePicker()
  const [dragging, setDragging] = useState(false)
  const { r, g, b } = hc
  const bg = `linear-gradient(90deg, rgba(255,255,255,0) 0%, rgba(${r},${g},${b},.5) 100%)`

  const stopDragging = () => {
    setDragging(false)
  }

  const handleDown = (e: React.TouchEvent | React.MouseEvent) => {
    e.preventDefault() // Prevent default behavior (like text selection or dragging)
    setDragging(true)
  }

  const handleOpacity = (e: React.TouchEvent | React.MouseEvent) => {
    const newO = getHandleValue(e) / 100
    const newColor = `rgba(${r}, ${g}, ${b}, ${newO})`
    handleChange(newColor)
  }

  const handleMove = (e: TouchEvent | MouseEvent) => {
    if (dragging) {
      handleOpacity(e as any)
    }
  }

  const handleClick = (e: React.TouchEvent | React.MouseEvent) => {
    if (!dragging) {
      handleOpacity(e)
    }
  }

  const left = squareWidth - 18

  useEffect(() => {
    const handleUp = () => {
      stopDragging()
    }

    // Add mouse and touch event listeners to the window (global)
    window.addEventListener('mousemove', handleMove)
    window.addEventListener('touchmove', handleMove)
    window.addEventListener('mouseup', handleUp)
    window.addEventListener('touchend', handleUp)

    return () => {
      window.removeEventListener('mousemove', handleMove)
      window.removeEventListener('touchmove', handleMove)
      window.removeEventListener('mouseup', handleUp)
      window.removeEventListener('touchend', handleUp)
    }
  }, [dragging]) // Re-run effect when 'dragging' state changes

  return (
    <div
      onMouseDown={handleDown}
      onTouchStart={handleDown}
      style={{
        height: 14,
        marginTop: 17,
        marginBottom: 4,
        cursor: 'ew-resize',
        position: 'relative',
      }}
    >
      <div
        style={{ ...defaultStyles.rbgcpCheckered, width: '100%', height: 14 }}
      />
      <div
        style={{ ...defaultStyles.rbgcpHandle, left: left * hc?.a, top: -2 }}
      />
      <div
        style={{ ...defaultStyles.rbgcpOpacityOverlay, background: bg }}
        onClick={(e) => handleClick(e)}
        onTouchStart={(e) => handleClick(e)}
      />
    </div>
  )
}

export default Opacity
